@@ -2,20 +2,26 @@ from psycopg2.extras import Inet
 
 from django.conf import settings
 from django.db.backends.base.operations import BaseDatabaseOperations
+from django.db.backends.utils import split_tzname_delta
 
 
 class DatabaseOperations(BaseDatabaseOperations):
-    cast_char_field_without_max_length = 'varchar'
-    explain_prefix = 'EXPLAIN'
+    cast_char_field_without_max_length = "varchar"
+    explain_prefix = "EXPLAIN"
     cast_data_types = {
-        'AutoField': 'integer',
-        'BigAutoField': 'bigint',
-        'SmallAutoField': 'smallint',
+        "AutoField": "integer",
+        "BigAutoField": "bigint",
+        "SmallAutoField": "smallint",
     }
 
     def unification_cast_sql(self, output_field):
         internal_type = output_field.get_internal_type()
-        if internal_type in ("GenericIPAddressField", "IPAddressField", "TimeField", "UUIDField"):
+        if internal_type in (
+            "GenericIPAddressField",
+            "IPAddressField",
+            "TimeField",
+            "UUIDField",
+        ):
             # PostgreSQL will resolve a union as type 'text' if input types are
             # 'unknown'.
             # https://www.postgresql.org/docs/current/typeconv-union-case.html
@@ -23,44 +29,50 @@ class DatabaseOperations(BaseDatabaseOperations):
             # PostgreSQL configuration so we need to explicitly cast them.
             # We must also remove components of the type within brackets:
             # varchar(255) -> varchar.
-            return 'CAST(%%s AS %s)' % output_field.db_type(self.connection).split('(')[0]
-        return '%s'
+            return (
+                "CAST(%%s AS %s)" % output_field.db_type(self.connection).split("(")[0]
+            )
+        return "%s"
 
     def date_extract_sql(self, lookup_type, field_name):
         # https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-EXTRACT
-        if lookup_type == 'week_day':
+        if lookup_type == "week_day":
             # For consistency across backends, we return Sunday=1, Saturday=7.
             return "EXTRACT('dow' FROM %s) + 1" % field_name
-        elif lookup_type == 'iso_week_day':
+        elif lookup_type == "iso_week_day":
             return "EXTRACT('isodow' FROM %s)" % field_name
-        elif lookup_type == 'iso_year':
+        elif lookup_type == "iso_year":
             return "EXTRACT('isoyear' FROM %s)" % field_name
         else:
             return "EXTRACT('%s' FROM %s)" % (lookup_type, field_name)
 
-    def date_trunc_sql(self, lookup_type, field_name):
+    def date_trunc_sql(self, lookup_type, field_name, tzname=None):
+        field_name = self._convert_field_to_tz(field_name, tzname)
         # https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-TRUNC
         return "DATE_TRUNC('%s', %s)" % (lookup_type, field_name)
 
     def _prepare_tzname_delta(self, tzname):
-        if '+' in tzname:
-            return tzname.replace('+', '-')
-        elif '-' in tzname:
-            return tzname.replace('-', '+')
+        tzname, sign, offset = split_tzname_delta(tzname)
+        if offset:
+            sign = "-" if sign == "+" else "+"
+            return f"{tzname}{sign}{offset}"
         return tzname
 
     def _convert_field_to_tz(self, field_name, tzname):
-        if settings.USE_TZ:
-            field_name = "%s AT TIME ZONE '%s'" % (field_name, self._prepare_tzname_delta(tzname))
+        if tzname and settings.USE_TZ:
+            field_name = "%s AT TIME ZONE '%s'" % (
+                field_name,
+                self._prepare_tzname_delta(tzname),
+            )
         return field_name
 
     def datetime_cast_date_sql(self, field_name, tzname):
         field_name = self._convert_field_to_tz(field_name, tzname)
-        return '(%s)::date' % field_name
+        return "(%s)::date" % field_name
 
     def datetime_cast_time_sql(self, field_name, tzname):
         field_name = self._convert_field_to_tz(field_name, tzname)
-        return '(%s)::time' % field_name
+        return "(%s)::time" % field_name
 
     def datetime_extract_sql(self, lookup_type, field_name, tzname):
         field_name = self._convert_field_to_tz(field_name, tzname)
@@ -71,12 +83,10 @@ class DatabaseOperations(BaseDatabaseOperations):
         # https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-TRUNC
         return "DATE_TRUNC('%s', %s)" % (lookup_type, field_name)
 
-    def time_trunc_sql(self, lookup_type, field_name):
+    def time_trunc_sql(self, lookup_type, field_name, tzname=None):
+        field_name = self._convert_field_to_tz(field_name, tzname)
         return "DATE_TRUNC('%s', %s)::time" % (lookup_type, field_name)
 
-    def json_cast_text_sql(self, field_name):
-        return '(%s)::text' % field_name
-
     def deferrable_sql(self):
         return " DEFERRABLE INITIALLY DEFERRED"
 
@@ -88,21 +98,30 @@ class DatabaseOperations(BaseDatabaseOperations):
         return cursor.fetchall()
 
     def lookup_cast(self, lookup_type, internal_type=None):
-        lookup = '%s'
+        lookup = "%s"
 
         # Cast text lookups to text to allow things like filter(x__contains=4)
-        if lookup_type in ('iexact', 'contains', 'icontains', 'startswith',
-                           'istartswith', 'endswith', 'iendswith', 'regex', 'iregex'):
-            if internal_type in ('IPAddressField', 'GenericIPAddressField'):
+        if lookup_type in (
+            "iexact",
+            "contains",
+            "icontains",
+            "startswith",
+            "istartswith",
+            "endswith",
+            "iendswith",
+            "regex",
+            "iregex",
+        ):
+            if internal_type in ("IPAddressField", "GenericIPAddressField"):
                 lookup = "HOST(%s)"
-            elif internal_type in ('CICharField', 'CIEmailField', 'CITextField'):
-                lookup = '%s::citext'
+            elif internal_type in ("CICharField", "CIEmailField", "CITextField"):
+                lookup = "%s::citext"
             else:
                 lookup = "%s::text"
 
         # Use UPPER(x) for case-insensitive lookups; it's faster.
-        if lookup_type in ('iexact', 'icontains', 'istartswith', 'iendswith'):
-            lookup = 'UPPER(%s)' % lookup
+        if lookup_type in ("iexact", "icontains", "istartswith", "iendswith"):
+            lookup = "UPPER(%s)" % lookup
 
         return lookup
 
@@ -127,29 +146,32 @@ class DatabaseOperations(BaseDatabaseOperations):
         # Perform a single SQL 'TRUNCATE x, y, z...;' statement. It allows us
         # to truncate tables referenced by a foreign key in any other table.
         sql_parts = [
-            style.SQL_KEYWORD('TRUNCATE'),
-            ', '.join(style.SQL_FIELD(self.quote_name(table)) for table in tables),
+            style.SQL_KEYWORD("TRUNCATE"),
+            ", ".join(style.SQL_FIELD(self.quote_name(table)) for table in tables),
         ]
         if reset_sequences:
-            sql_parts.append(style.SQL_KEYWORD('RESTART IDENTITY'))
+            sql_parts.append(style.SQL_KEYWORD("RESTART IDENTITY"))
         if allow_cascade:
-            sql_parts.append(style.SQL_KEYWORD('CASCADE'))
-        return ['%s;' % ' '.join(sql_parts)]
+            sql_parts.append(style.SQL_KEYWORD("CASCADE"))
+        return ["%s;" % " ".join(sql_parts)]
 
     def sequence_reset_by_name_sql(self, style, sequences):
         # 'ALTER SEQUENCE sequence_name RESTART WITH 1;'... style SQL statements
         # to reset sequence indices
         sql = []
         for sequence_info in sequences:
-            table_name = sequence_info['table']
+            table_name = sequence_info["table"]
             # 'id' will be the case if it's an m2m using an autogenerated
             # intermediate table (see BaseDatabaseIntrospection.sequence_list).
-            column_name = sequence_info['column'] or 'id'
-            sql.append("%s setval(pg_get_serial_sequence('%s','%s'), 1, false);" % (
-                style.SQL_KEYWORD('SELECT'),
-                style.SQL_TABLE(self.quote_name(table_name)),
-                style.SQL_FIELD(column_name),
-            ))
+            column_name = sequence_info["column"] or "id"
+            sql.append(
+                "%s setval(pg_get_serial_sequence('%s','%s'), 1, false);"
+                % (
+                    style.SQL_KEYWORD("SELECT"),
+                    style.SQL_TABLE(self.quote_name(table_name)),
+                    style.SQL_FIELD(column_name),
+                )
+            )
         return sql
 
     def tablespace_sql(self, tablespace, inline=False):
@@ -160,46 +182,36 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def sequence_reset_sql(self, style, model_list):
         from django.db import models
+
         output = []
         qn = self.quote_name
         for model in model_list:
-            # Use `coalesce` to set the sequence for each model to the max pk value if there are records,
-            # or 1 if there are none. Set the `is_called` property (the third argument to `setval`) to true
-            # if there are records (as the max pk value is already in use), otherwise set it to false.
-            # Use pg_get_serial_sequence to get the underlying sequence name from the table name
-            # and column name (available since PostgreSQL 8)
+            # Use `coalesce` to set the sequence for each model to the max pk
+            # value if there are records, or 1 if there are none. Set the
+            # `is_called` property (the third argument to `setval`) to true if
+            # there are records (as the max pk value is already in use),
+            # otherwise set it to false. Use pg_get_serial_sequence to get the
+            # underlying sequence name from the table name and column name.
 
             for f in model._meta.local_fields:
                 if isinstance(f, models.AutoField):
                     output.append(
                         "%s setval(pg_get_serial_sequence('%s','%s'), "
-                        "coalesce(max(%s), 1), max(%s) %s null) %s %s;" % (
-                            style.SQL_KEYWORD('SELECT'),
+                        "coalesce(max(%s), 1), max(%s) %s null) %s %s;"
+                        % (
+                            style.SQL_KEYWORD("SELECT"),
                             style.SQL_TABLE(qn(model._meta.db_table)),
                             style.SQL_FIELD(f.column),
                             style.SQL_FIELD(qn(f.column)),
                             style.SQL_FIELD(qn(f.column)),
-                            style.SQL_KEYWORD('IS NOT'),
-                            style.SQL_KEYWORD('FROM'),
+                            style.SQL_KEYWORD("IS NOT"),
+                            style.SQL_KEYWORD("FROM"),
                             style.SQL_TABLE(qn(model._meta.db_table)),
                         )
                     )
-                    break  # Only one AutoField is allowed per model, so don't bother continuing.
-            for f in model._meta.many_to_many:
-                if not f.remote_field.through:
-                    output.append(
-                        "%s setval(pg_get_serial_sequence('%s','%s'), "
-                        "coalesce(max(%s), 1), max(%s) %s null) %s %s;" % (
-                            style.SQL_KEYWORD('SELECT'),
-                            style.SQL_TABLE(qn(f.m2m_db_table())),
-                            style.SQL_FIELD('id'),
-                            style.SQL_FIELD(qn('id')),
-                            style.SQL_FIELD(qn('id')),
-                            style.SQL_KEYWORD('IS NOT'),
-                            style.SQL_KEYWORD('FROM'),
-                            style.SQL_TABLE(qn(f.m2m_db_table()))
-                        )
-                    )
+                    # Only one AutoField is allowed per model, so don't bother
+                    # continuing.
+                    break
         return output
 
     def prep_for_iexact_query(self, x):
@@ -221,9 +233,9 @@ class DatabaseOperations(BaseDatabaseOperations):
     def distinct_sql(self, fields, params):
         if fields:
             params = [param for param_list in params for param in param_list]
-            return (['DISTINCT ON (%s)' % ', '.join(fields)], params)
+            return (["DISTINCT ON (%s)" % ", ".join(fields)], params)
         else:
-            return ['DISTINCT'], []
+            return ["DISTINCT"], []
 
     def last_executed_query(self, cursor, sql, params):
         # https://www.psycopg.org/docs/cursor.html#cursor.query
@@ -234,14 +246,16 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def return_insert_columns(self, fields):
         if not fields:
-            return '', ()
+            return "", ()
         columns = [
-            '%s.%s' % (
+            "%s.%s"
+            % (
                 self.quote_name(field.model._meta.db_table),
                 self.quote_name(field.column),
-            ) for field in fields
+            )
+            for field in fields
         ]
-        return 'RETURNING %s' % ', '.join(columns), ()
+        return "RETURNING %s" % ", ".join(columns), ()
 
     def bulk_insert_sql(self, fields, placeholder_rows):
         placeholder_rows_sql = (", ".join(row) for row in placeholder_rows)
@@ -257,13 +271,16 @@ class DatabaseOperations(BaseDatabaseOperations):
     def adapt_timefield_value(self, value):
         return value
 
+    def adapt_decimalfield_value(self, value, max_digits=None, decimal_places=None):
+        return value
+
     def adapt_ipaddressfield_value(self, value):
         if value:
             return Inet(value)
         return None
 
     def subtract_temporals(self, internal_type, lhs, rhs):
-        if internal_type == 'DateField':
+        if internal_type == "DateField":
             lhs_sql, lhs_params = lhs
             rhs_sql, rhs_params = rhs
             params = (*lhs_params, *rhs_params)
@@ -274,15 +291,21 @@ class DatabaseOperations(BaseDatabaseOperations):
         prefix = super().explain_query_prefix(format)
         extra = {}
         if format:
-            extra['FORMAT'] = format
+            extra["FORMAT"] = format
         if options:
-            extra.update({
-                name.upper(): 'true' if value else 'false'
-                for name, value in options.items()
-            })
+            extra.update(
+                {
+                    name.upper(): "true" if value else "false"
+                    for name, value in options.items()
+                }
+            )
         if extra:
-            prefix += ' (%s)' % ', '.join('%s %s' % i for i in extra.items())
+            prefix += " (%s)" % ", ".join("%s %s" % i for i in extra.items())
         return prefix
 
     def ignore_conflicts_suffix_sql(self, ignore_conflicts=None):
-        return 'ON CONFLICT DO NOTHING' if ignore_conflicts else super().ignore_conflicts_suffix_sql(ignore_conflicts)
+        return (
+            "ON CONFLICT DO NOTHING"
+            if ignore_conflicts
+            else super().ignore_conflicts_suffix_sql(ignore_conflicts)
+        )
